function v_msg() {
    $NONVERBOSE || echo "$ARGV0: $*" > /dev/stderr
}

# ldap_cmd <ldaptools-executable> <arguments> [..]
function ldap_cmd() {
    local CMD=$1
    shift
    local ARGS=${LDAP_EXTRA_ARGS-}
    [ -z "${LDAP_URI+x}" ] || ARGS="$ARGS -H $LDAP_URI"
    [ -z "${LDAP_BINDDN+x}" ] || ARGS="$ARGS -D $LDAP_BINDDN"
    v_msg "ldap_cmd: $DRYRUN $CMD $ARGS $*"
    $DRYRUN $CMD $ARGS $*
}

# config_read <outvar> <key> [ <section> ]
function config_read() {
    case $# in
	2) bashini_get_value "$1" config "$2";;
	3) bashini_get_value_with_default "$1" config "$2" "$3";;
	*) fatal "config_read_optional called with $# args instead of 2 or 3";;
    esac
}

# config_read_optional <outvar> <key> [ <section> ]
function config_read_optional() {
	config_read "$1" "$2" "${3-}" || unset "$1"
}

# config_read_mandatory <outvar> <key> [ <section> ]
function config_read_mandatory() {
	config_read "$1" "$2" "$3" || fatal "mandatory config option '$2' missing"
}

# config_read_all [ <section> ]
function config_read_common() {
    local SECTION="${1-}"

    # LDAP access
    config_read_optional LDAP_URI "ldap uri" "$SECTION"
    config_read_optional LDAP_BINDDN "ldap bind dn" "$SECTION"
    config_read_optional LDAP_EXTRA_ARGS "ldap extra args" "$SECTION"

    # LDAP tree config
    config_read_mandatory LDAP_SEARCH_BASE "ldap search base" "$SECTION"
    config_read_optional LDAP_USER_SUBHIER "ldap user subhier" "$SECTION"
    config_read_optional LDAP_GROUP_SUBHIER "ldap group subhier" "$SECTION"
    config_read_optional LDAP_AUTOFS_SUBHIER "ldap autofs subhier" "$SECTION"
    config_read_optional LDAP_AUTOFS_HOMES "ldap autofs homes" "$SECTION"

    # behaviour
    config_read WINDOWS_USERS "windows users" $SECTION || WINDOWS_USERS=false
    config_read AUTOFS_USERS "windows users" $SECTION || WINDOWS_USERS=false
    config_read_optional AUTOFS_HOMES_PREFIX "autofs homes prefix" $SECTION
}

# launch cmd [ args [..] ]
#  launches other command of ldapusermgmt in same location
#  and honors common args
function launch() {
    local CMD="$1"
    shift

    # fixme, that eval is most probably useless
    # check: will that work with spaces in common_args?
    v_msg "launching $CMD $COMMON_ARGS $*"
    eval $CMD $COMMON_ARGS $*
}
